{{ audio_oscillator.spin }}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 6_144_000


  mclkpin  = 14
  bclkpin  = 15
  lrclkpin = 16
  dinpin   = 17

  rotaryA  = 25
  rotaryPush = 26
  rotaryB  = 27

  samplerate = 48000
  hertz      = 89478
  hertzfract = 2096000000
  millihertz = 89
  millihertzfract = 2055078844
  ratio =  samplerate * 1000
  maxfreq = 20000*1000

  propTX  = 30 'programming output
  propRX  = 31 'programming input
  baudrate = 57600


OBJ
  Clock: "Clock"
  debug: "SerialMirror"
  fmt: "Format"
  DAC: "WM8524_DAC"
  cordic: "cordic"
  SevenSeg : "7seg-dual-charlie"

VAR
  BYTE buffer[100] 'buffer to assemble output strings

  'long  pinassign  ' parms for DAC
  'long  left_addr
  'long  right_addr


  long  left	   ' parms for cordic
  long  right
  long  frequency
  long  waveform

  long  temp
  long  prev
  long  diff
  long  n
  long  stepp

  long  hertzfreq
  long  dppos
  long  steppow
  long  i

PUB Main
  Clock.init(6_144_000)
  Clock.setclock(XTAL1 + PLL16X) 'Set clock to really fast
  debug.start (10,11,0,baudrate)
  prev := 0
  steppow := 2+3
  dppos := 3
  stepp := 1
  repeat i from 1 to steppow
    stepp *= 10

  left := 0
  right := 0
  hertzfreq := 400000
  frequency := hertzfreq * millihertz + hertzfreq ** millihertzfract
  waveform := 0

  'pinassign := mclkpin | (bclkpin << 8) | (lrclkpin << 16) | (dinpin << 24)
  DAC.Start (mclkpin, bclkpin, lrclkpin, dinpin, @left, @right)
  cordic.Begin (@left)
  SevenSeg.Start (@hertzfreq)
  n := 0
  repeat
    { read the rotary encoder }
    temp := (ina >> rotaryA) & 1
    temp := (temp << 1) | ((ina >> rotaryB) & 1)
    { convert from gray to binary 2-bit code }
    temp := temp ^ (temp >> 1)
    { move to where sign easy to test, calculate difference from last time }
    temp := temp << 30
    diff := temp - prev
    prev := temp
    temp >>= 30
    if temp == 2
      { now diff is normally 0, but if not sign gives step direction }
      if ina[rotaryPush]
        { push button down, change step rate }
        if diff > 0
          steppow -= 1
          if steppow < 0
            steppow := 0
        if diff < 0
          steppow += 1
          if steppow > 7
            steppow := 7
        if (diff <> 0)
          stepp := 1
          if steppow > 0
            repeat i from 1 to steppow
              stepp *= 10
      else
        { step the frequency value }
        if diff < 0
          hertzfreq -= stepp
          if hertzfreq < 0 or hertzfreq > maxfreq
            hertzfreq := 0
        if diff > 0
          hertzfreq += stepp
          if hertzfreq < 0 or hertzfreq > maxfreq
            hertzfreq := maxfreq

    frequency := hertzfreq * millihertz + hertzfreq ** millihertzfract

    if (n & $FF) == 0
      fmt.sprintf (@buffer,string("f=%iHz  "),frequency/Hertz)
      debug.str (@buffer)
      fmt.sprintf (@buffer,string("s=%iHz  "),stepp/Hertz)
      debug.str (@buffer)
      debug.tx (13)
    n += 1
    {{
    waitcnt (cnt+1000000)
    temp := waveform + 1
    if (temp > 2)
      temp := 0
    waveform := temp
    }}





